#!/bin/ksh
###############################################################################
# Licensed Materials - Property of IBM
# 5724-T40
# 5724-S94
#
# (C) Copyright IBM Corp. 2009.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################################

init_logging() {
	if [ "${TRACE_FILE:+LogsAlreadyConfigured}" = "LogsAlreadyConfigured" ] ; then
		return 0
	fi	

	# Tivoli Common Dir settings
	TCD_LOG_PROPERTIES=/etc/ibm/tivoli/common/cfg/log.properties
	TCD_LOCATION=/var/ibm/tivoli/common
	TRACE_FILE_NAME=ILMT-TAD4D-agent-
	TRACE_FILE_EXT=.trc
	MSG_FILE_NAME=ILMT-TAD4D-agent-
	MSG_FILE_EXT=.msg

	# Temporary logs configuration
	if [ -n "$1" ] ; then
		LOG_TEMP_DIR=$1
	else
		LOG_TEMP_DIR=/tmp
	fi
	FILE_TIMESTAMP=`date '+%Y%m%d-%H%M'`
	MSG_FILE=${LOG_TEMP_DIR}/${MSG_FILE_NAME}${FILE_TIMESTAMP}${MSG_FILE_EXT}
	TRACE_FILE=${LOG_TEMP_DIR}/${TRACE_FILE_NAME}${FILE_TIMESTAMP}${TRACE_FILE_EXT}
	touch "$MSG_FILE"
	touch "$TRACE_FILE"

	trace "Entering init_logging()"
	# Checking existence of Tivoli Common Dir
	if [ -f "$TCD_LOG_PROPERTIES" ] ; then
		trace "TCD log.properties found."
		TCD_LOCATION_TMP=`cat ${TCD_LOG_PROPERTIES}`
		TCD_LOCATION=`echo "$TCD_LOCATION_TMP" | sed 's/.*=//'`
		trace "TCD location is $TCD_LOCATION"
	else
		trace "TCD log.properties not found. Creating TCD in default unix location: $TCD_LOCATION"
	fi
	
	MSG_DIR=$TCD_LOCATION/COD/logs/install/message
	TRACE_DIR=$TCD_LOCATION/COD/logs/install/trace
	
	if [ ! -d "$MSG_DIR" ] ; then
		trace "Creating directory: $MSG_DIR"
		mkdir -p "$MSG_DIR"
	fi
	if [ ! -d "$TRACE_DIR" ] ; then
		trace "Creating directory: $TRACE_DIR"
		mkdir -p "$TRACE_DIR"
	fi

	mv "$MSG_FILE" "$MSG_DIR"
	mv "$TRACE_FILE" "$TRACE_DIR"

	MSG_FILE=${MSG_DIR}/${MSG_FILE_NAME}${FILE_TIMESTAMP}${MSG_FILE_EXT}
	export MSG_FILE
	TRACE_FILE=${TRACE_DIR}/${TRACE_FILE_NAME}${FILE_TIMESTAMP}${TRACE_FILE_EXT}
	export TRACE_FILE

	trace "Exiting init_logging()"	
}

# This fuction is used to log traces in the trace file. Traces are not globalized.
trace() {
	TIMESTAMP=`date '+%Y-%m-%dT%H:%M:%S'`	
	echo "$TIMESTAMP $1" >> "$TRACE_FILE"
}

# Only globalized messages for the user are logged by this fuction. Messages should
# provide overall information about installation status, should not mention about
# installation package internal parts (scripts, filesets, etc.)
msg() {
	TIMESTAMP=`date '+%Y-%m-%dT%H:%M:%S'`	
	message=`${PARAMS[AGT_TEMP_DIR]}/nls/nls_replace $*`
	echo "$TIMESTAMP $message" >> "$MSG_FILE"
}
