#!/bin/ksh
###############################################################################
# IBM Confidential
#
# OCO Source Materials
#
# 5724-T40
# 5724-S94
#
# (C) Copyright IBM Corp. 2009
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#
###############################################################################

# Agent parameters mapped to indices that are used later
# to address the parameters array.
AGT_TEMP_DIR=1
AGT_PKG_DIR=2
URI=3
ORGANIZATION=4
INSTALL_AGENT=5
PORT=6
SECURE_PORT=7
CLIENT_AUTH_SECURE_PORT=8
AGT_INSTALL_PATH=9
AGT_TEMP_PATH=10
SERVER_ADDRESS=11
SCAN_GROUP=12
SECURITY_LEVEL=13
USE_PROXY=14
PROXY_PORT=15
PROXY_NAME=16
FIPS_ENABLED=17
MAX_CACHE_SIZE=18
CIT_INSTALL_PATH=19
INSTALL_CERTIFICATE=20
CUSTOM_CERT=21
CERT_PATH=22
PRIVATE_CERT=23
SYSTEM_ACTIVE_PROCESSORS=24
SHARED_POOL_CAPACITY=25
PROCESSOR_TYPE=26
DSD=27
DISABLE_ROLLBACK=28
MACHINE_TYPE=29

# An array that keeps the agent configuration. It is initalized
# by default values. 
set -A PARAMS

# Default installation settings that cannot be set in response file
PARAMS[AGT_TEMP_DIR]=/tmp/itlm
PARAMS[AGT_PKG_DIR]=/tmp/itlm/pkg
PARAMS[URI]=/msghandler/service
PARAMS[ORGANIZATION]=DEFAULT_CUSTOMER
PARAMS[INSTALL_AGENT]=yes

# Default installation settings that can be set in response file
PARAMS[PORT]=9988
PARAMS[SECURE_PORT]=9999
PARAMS[CLIENT_AUTH_SECURE_PORT]=9977
PARAMS[AGT_INSTALL_PATH]=/var/itlm
PARAMS[AGT_TEMP_PATH]=/tmp/itlm
PARAMS[SERVER_ADDRESS]=localhost
PARAMS[SCAN_GROUP]=DEFAULT
PARAMS[SECURITY_LEVEL]=0
PARAMS[USE_PROXY]=n
PARAMS[PROXY_PORT]=3128
PARAMS[PROXY_NAME]=none
PARAMS[FIPS_ENABLED]=n
PARAMS[MAX_CACHE_SIZE]=2097152
PARAMS[CIT_INSTALL_PATH]=
PARAMS[INSTALL_CERTIFICATE]=no
PARAMS[CUSTOM_CERT]=false
PARAMS[CERT_PATH]=
PARAMS[SYSTEM_ACTIVE_PROCESSORS]=
PARAMS[SHARED_POOL_CAPACITY]=
PARAMS[PROCESSOR_TYPE]=IFL
PARAMS[MACHINE_TYPE]=z9
PARAMS[DSD]=n
PARAMS[PRIVATE_CERT]=none
PARAMS[DISABLE_ROLLBACK]=

# Agent configuration files
AGT_INSTALL_IN=_agent_config.txt
AGT_INSTALL_OUT=agent_install.txt

# Default location and name for response file
RESPONSE_FILE_PATH=/etc
RESPONSE_FILE_NAME=response_file.txt

#disableRollBack param name
DIS_ROLL_BACK_NAME=disableRollBack

# Default (already installed) agent configuration files
AGT_INI_FILE=/etc/tlmagent.ini
AGT_SUBCAPACITY_FILE=/etc/tlmsubcapacity.cfg

# File that is created to mark that this is an upgrade or refresh scenario
AGT_INSTALLED_FILE=${PARAMS[AGT_TEMP_DIR]}/.agtinstalled

#Embedded certificate path (test certificate file included in installation package)
EMBEDDED_CERT_PATH=${PARAMS[AGT_PKG_DIR]}/cert.arm

# Function checks if agent is already installed. It is determined by existence of AGT_INI_FILE.
# If the file is present in the filesystem, marker file AGT_INSTALLED_FILE is created. The point is
# that some native installation mechanisms (i.e installp on AIX) does rollback automatically in case of
# error, which effects in invoking uninstallation script(s). In order to prevent uninstalling old version of agent
# by uninstallation script marker file is created in upgrade or refresh scenario.
is_agent_installed() {
	if [ -f "$AGT_INSTALLED_FILE" ] ; then
		trace "Removing old $AGT_INSTALLED_FILE file"
		rm -f "$AGT_INSTALLED_FILE"
	fi

	if [ -f "$AGT_INI_FILE" ] ; then
		trace "Existing agent installation detected: agent ini file found in $AGT_INI_FILE"
		trace "Creating temporary file $AGT_INSTALLED_FILE"
		touch "$AGT_INSTALLED_FILE"		
		return 1
	else
		trace "Existing agent installation not detected: agent ini file not found in $AGT_INI_FILE"
		return 0
	fi
}

is_agent_installed_cleanup() {
	if [ -f "$AGT_INSTALLED_FILE" ] ; then
		trace "Removing temporary file $AGT_INSTALLED_FILE"
		rm -f "$AGT_INSTALLED_FILE"
	else 
		trace "Temporary file $AGT_INSTALLED_FILE not found"
	fi
}

resolve_agt_install_in() {
	trace "Entering resolve_agt_install_in()"
	AGT_INSTALL_IN=$(ls ${PARAMS[AGT_PKG_DIR]} | grep $AGT_INSTALL_IN)
	trace "Input configuration file name resolved as \"$AGT_INSTALL_IN\""
	if [ ! -f "${PARAMS[AGT_PKG_DIR]}/$AGT_INSTALL_IN" ] ; then
		#This really should not occur unless the installation package is prepared in a wrong way.
		trace "Error: Input configuration file not found. Installation failed."
		installation_failed;
	fi
	trace "Exiting resolve_agt_install_in()"	
}

# In below function there are parameters' names from agent configuration
# file and from agent response file mapped to indeces from parameters array
map_parameters() {
	# Parameters that cannot be set in response file. The mapping is done only for
	# parameters for agent configuration file.
	#agt_temp_dir & agt_pkg_dir should point to the same directory otherwise slmerr generated by swd will not be found
	agt_temp_dir=$AGT_PKG_DIR
	agt_pkg_dir=$AGT_PKG_DIR
	uri=$URI
	organization=$ORGANIZATION
	installAgent=$INSTALL_AGENT
	privateCertificate=$PRIVATE_CERT

	# Parameters that can be set in response file. That's why there are two mappings:
	# one for response file parameters and one for agent configuration file parameters
	# (file that is passed to installagent binary)
	port=$PORT
	Port=$PORT
	
	# this means that in agent configuration file we need resolve "secure_port" string...
	secure_port=$SECURE_PORT
	#... but we want slightly redefine the phrase, that we expect in response file
	SecureAuth=$SECURE_PORT
	
	client_auth_secure_port=$CLIENT_AUTH_SECURE_PORT
	SecureAll=$CLIENT_AUTH_SECURE_PORT

	agt_install_path=$AGT_INSTALL_PATH
	AgentInstallPath=$AGT_INSTALL_PATH
	
	agt_tmp_path=$AGT_TEMP_PATH
	AgentTempPath=$AGT_TEMP_PATH
	
	serverAddress=$SERVER_ADDRESS
	MessageHandlerAddress=$SERVER_ADDRESS
	
	scanGroup=$SCAN_GROUP
	ScanGroup=$SCAN_GROUP
	
	security_level=$SECURITY_LEVEL
	SecurityLevel=$SECURITY_LEVEL
	
	useProxy=$USE_PROXY
	UseProxy=$USE_PROXY
	
	proxyPort=$PROXY_PORT
	ProxyPort=$PROXY_PORT
	
	proxyName=$PROXY_NAME
	ProxyAddress=$PROXY_NAME
	
	fips_enabled=$FIPS_ENABLED
	FipsEnabled=$FIPS_ENABLED
	
	max_cache_size=$MAX_CACHE_SIZE
	MaxCacheSize=$MAX_CACHE_SIZE
	
	CIT_DestinationDirectory=$CIT_INSTALL_PATH
	CITInstallPath=$CIT_INSTALL_PATH
	
	# Parameter that determines if certificate file should be installed. Possible values: 'yes', 'no'
	# If set to yes, then the next parameter (CustomSSLCertificate) is important
	installCertificate=$INSTALL_CERTIFICATE
	InstallCertificate=$INSTALL_CERTIFICATE
	
	# Parameter that we expect only in response file. It does not exist in agent configuration file. Possible values 'true' or 'false'.
	CustomSSLCertificate=$CUSTOM_CERT
	
	certificate_path=$CERT_PATH
	CertFilePath=$CERT_PATH
	
	systemActiveProcessors=$SYSTEM_ACTIVE_PROCESSORS
	SystemActiveProcessors=$SYSTEM_ACTIVE_PROCESSORS
	
	sharedPoolCapacity=$SHARED_POOL_CAPACITY
	SharedPoolCapacity=$SHARED_POOL_CAPACITY
	
	processorType=$PROCESSOR_TYPE
	ProcessorType=$PROCESSOR_TYPE

	machineType=$MACHINE_TYPE
	MachineType=$MACHINE_TYPE
	
	dsd=$DSD
	InstallOnDD=$DSD
	
	disableRollBack=$DISABLE_ROLLBACK
}

resolve_conf() {
	trace "Entering resolve_conf()"
	resolve_agt_install_in;
	AGT_INSTALL_OUT=${PARAMS[AGT_PKG_DIR]}/$AGT_INSTALL_OUT
	
	if [ -f "$AGT_INSTALL_OUT" ] ; then
		trace "Removing old $AGT_INSTALL_OUT file"
		rm -f "$AGT_INSTALL_OUT"
	fi
	
	typeset pattern='=\$(..*)'	
	{ while read fline ; do 		
		containsPattern=$(echo $fline | grep $pattern)
		if [ -n "$containsPattern" ] ; then
			typeset property=$(echo $fline | sed 's|\(..*=$(\)\(..*\)\()\)|\2|')
			echo $fline | sed "s|\$($property)|${PARAMS[property]}|" >> "$AGT_INSTALL_OUT"
				
			if [ $IS_AGENT_INSTALLED = 0 ] ; then
			trace "Resolving \$($property) with ${PARAMS[property]}"			
			fi			
		else
			echo $fline >> "$AGT_INSTALL_OUT"
		fi
	done } < "${PARAMS[AGT_PKG_DIR]}/$AGT_INSTALL_IN"
	trace "Exiting resolve_conf()"	
}

validate_param_name() {
	typeset param=$1
	typeset isEmpty=$(set | grep "^$1=")
	if [ -z "$isEmpty" ] ; then
		trace "Parameter \"$param\" provided in response file not recognized."
		return 1
	else
		return 0
	fi	
}

load_response() {
	trace "Starting load_response()"
	trace "Loading parameters values from $LMT_RESPONSE_FILE_PATH"
	typeset CR_CHAR=$(echo -e "\r")
	#some implementations of echo dont recognize -e option and the above return something like -e^M, in this case we should use echo w/o -e switch
	typeset ESWITCH_TEST=$(echo $CR_CHAR | grep 'e')
	if [ -n "$ESWITCH_TEST" ] ; then
		CR_CHAR=$(echo "\r")
	fi
	{ while read fline ; do 
		#Convert end of line to unix line if needed
		typeset CR_END=$(echo $fline | grep "${CR_CHAR}$")
		if [ "$CR_END" != "" ] ; then
			fline=$(echo $fline | sed 's|.$||')
		fi
		# Skip comment or empty line
		typeset isComment=$(echo $fline | grep "^[ \t]*#")
		typeset isEmpty=$(echo $fline | sed 's/^[ \t]*//;s/[ \t]*$//')
		if [[ -n "$isComment" || -z "$isEmpty" ]] ; then
			continue
		fi
		typeset property=$(echo $fline | sed 's|=.*||')
		validate_param_name $property
		if [ $? = 1 ] ; then
			trace "Skipping improper parameter \"$property\"."
			msg "nativeAgt.wrongResponseParam" "\"$property\""
			continue
		fi
		
		if [[ -z "$1" || -n "$1" && "$1" = "$property" ]]
		then
			typeset value=$(echo $fline | sed 's|\(.*\)\(=\)\(.*\)|\3|')
			if [ "$value" = "" ] ; then
				trace "Parameter's $property value is blank. Setting the default value."
			else
				trace "Reading: ${property} = ${value}."
				PARAMS[property]=$value
			fi
		else
			trace "Ignoring parameter ${property}."
		fi
	done } < "$LMT_RESPONSE_FILE_PATH"
	trace "Exiting load_response()"
}

# Function checks if agent's parameters should be loaded from response file.
# Fuction returns:
# 0 - if variable LMT_RESPONSE_FILE_PATH is not set and the response
#      file with the default name and in default location cannot be found 
# 1 - if variable LMT_RESPONSE_FILE_PATH points to existing response file
#      or the response file was found with the default name and in default location
#
# In case the variable LMT_RESPONSE_FILE_PATH is set, but points 
# to non-existing response file, the function calls exit 1 which results 
# in agent installation failure.
is_inst_from_response() {
	trace "Entering is_inst_from_response()"
	INST_FROM_RESPONSE=${LMT_RESPONSE_FILE_PATH:+1}
	if [ "$INST_FROM_RESPONSE" = "1" ] ; then
		trace "LMT_RESPONSE_FILE_PATH set to ${LMT_RESPONSE_FILE_PATH}."
		if [ -f "$LMT_RESPONSE_FILE_PATH" ] ; then 			
			trace "Exiting inst_from_response()"
			return 1
		else
			trace "Error: Missing response file in specified location: LMT_RESPONSE_FILE_PATH=${LMT_RESPONSE_FILE_PATH}. Installation failed."
			msg "nativeAgt.responseNotFound" "LMT_RESPONSE_FILE_PATH=$LMT_RESPONSE_FILE_PATH"
			installation_failed;
		fi
	else
		trace "LMT_RESPONSE_FILE_PATH not set. Looking for default response file..."
		if [ -f "$RESPONSE_FILE_PATH/$RESPONSE_FILE_NAME" ] ; then
			trace "Response file with default name in default location ($RESPONSE_FILE_PATH/$RESPONSE_FILE_NAME) found."
			LMT_RESPONSE_FILE_PATH=$RESPONSE_FILE_PATH/$RESPONSE_FILE_NAME
			trace "Exiting is_inst_from_response()"			
			return 1
		else
			trace "Response file with default name in default location ($RESPONSE_FILE_PATH/$RESPONSE_FILE_NAME) not found."
			trace "Exiting is_inst_from_response()"
			return 0
		fi
	fi
}

checkTrueOrFalse() {
	typeset -l param=$1
	case "$param" in
		*yes*|*y*|*true*) return 1;;
		*) return 0;;
	esac
}

prepare_install_agent() {
	trace "Entering prepare_install_agent()"

	if [ $INST_FROM_RESPONSE = 1 ] ; then
		if [ "${PARAMS[DISABLE_ROLLBACK]}" = "" ] ; then
			trace "disableRollBack parameter not set"
		else
			trace "disableRollBack parameter set to ${PARAMS[DISABLE_ROLLBACK]}"
			export disableRollBack=${PARAMS[DISABLE_ROLLBACK]}
		fi
	fi

	checkTrueOrFalse "${PARAMS[INSTALL_CERTIFICATE]}";
	typeset INST_CERT=$?
	if [ "$INST_CERT" = "0" ] ; then
		PARAMS[CERT_PATH]=""
		trace "No certificate file will be installed."
	else
		checkTrueOrFalse "${PARAMS[CUSTOM_CERT]}";
		typeset CUST_CERT=$?
		if [ "$CUST_CERT" = "0" ] ; then
			PARAMS[CERT_PATH]=$EMBEDDED_CERT_PATH
			trace "Embedded (test) certificate file will be installed."
		else
			if [ ! -f "${PARAMS[CERT_PATH]}" ] ; then
				trace "Error: Specified certificate file ${PARAMS[CERT_PATH]} could not be found."
				installation_failed;
			else
				trace "Custom certificate file will be installed from ${PARAMS[CERT_PATH]}"
			fi
		fi
	fi
	
	trace "Exiting prepare_install_agent()"
}

install_agent() {
	trace "Entering install_agent()"
	trace "Invoking installagent..."
		
	${PARAMS[AGT_PKG_DIR]}/installagent ${PARAMS[AGT_PKG_DIR]} -f ${PARAMS[AGT_PKG_DIR]}/agent_install.txt
	
	typeset SLMRC_FILE=${PARAMS[AGT_PKG_DIR]}/slmrc
	if [ -f "$SLMRC_FILE" ] ; then
		typeset SLMRC=$(cat "$SLMRC_FILE")
		trace "Installagent return code from $SLMRC_FILE is: ${SLMRC}"
	else
		trace "Error: Could not find file with installagent returcode: ${SLMRC_FILE}. Installation failed."
		installation_failed;
	fi
	
	# The following error codes denote successfull installation:
	# 0  - installation successfull
	# 30 - installation successfull but gskit requires system reboot
	# 31 - installation successfull but agent was not started because of missing gskit prerequisites 
	if [[ "$SLMRC" != "0" && "$SLMRC" != "30" && "$SLMRC" != "31" ]] ; then
		trace "Error: Installagent returned $SLMRC. Installation failed."
		installation_failed;
	fi
		
	trace "Exiting install_agent()"
}

precheck_linux_s390() {
	trace "Entering precheck_linux_s390()"
	if [ $IS_AGENT_INSTALLED = 1 ] ; then
		trace "Looking for $AGT_SUBCAPACITY_FILE"
		if [ ! -f "$AGT_SUBCAPACITY_FILE" ] ; then
			trace "Error: $AGT_SUBCAPACITY_FILE not found. Installation failed."
			installation_failed;
		fi
	fi
	trace "Exiting precheck_linux_s390()"	
}

postcheck_linux_s390() {
	trace "Entering postcheck_linux_s390()"
	if [ $IS_AGENT_INSTALLED != 1 ] ; then
	  typeset missingParameter="false"
	  if [ "${PARAMS[SYSTEM_ACTIVE_PROCESSORS]}" = "" ] ; then
		  trace "Error: parameter \"SystemActiveProcessors\" not set. Use response file in order to set the missing value."
		  msg "nativeAgt.missingParam" "SystemActiveProcessors"
		  missingParameter="true"
	  fi
	  if [ "${PARAMS[SHARED_POOL_CAPACITY]}" = "" ] ; then
		  trace "Error: parameter \"SharedPoolCapacity\" not set. Use response file in order to set the missing value."
		  msg "nativeAgt.missingParam" "SharedPoolCapacity"
		  missingParameter="true"
	  fi
	  if [ "$missingParameter" = "true" ] ; then
		  trace "Error: At least one obligatory parameter not set. Installation failed."
		  installation_failed;
	  fi
  fi
	trace "Exiting postcheck_linux_s390()"
}

check_platform_conditions() {
	trace "Entering check_platform_conditions()"

	typeset FUNCTION_MODE=$1
	trace "Function mode is $FUNCTION_MODE"

	typeset OS_NAME=$(uname -s)
	case $OS_NAME in
		Linux) 	{
					typeset PROC_TYPE_S390=$(uname -p | grep s390)
					if [ -n "$PROC_TYPE_S390" ] ; then
						trace "Checking ${FUNCTION_MODE}requisites for $OS_NAME $PROC_TYPE_S390"
						if [ "$FUNCTION_MODE" = "pre" ] ; then
							precheck_linux_s390;
						else
							postcheck_linux_s390;
						fi
					fi
				};;
			*)	;;
	esac	
	trace "Exiting check_platform_conditions()"
}

installation_failed() {
	msg "nativeAgt.installationFailed"
	
	if [ $IS_AGENT_INSTALLED = 1 ] ; then
		trace "Upgrade/Refresh failed"
		is_agent_installed_cleanup;
		exit 0
	fi
	
	exit 1
}


main() {
	trace "Entering main()"
	
	#Check at the very beginning if there's agent currently installed
	is_agent_installed;
	IS_AGENT_INSTALLED=$?	
	
	map_parameters;
	
	#Check if installation is supposed to relay on response file
	is_inst_from_response;
	INST_FROM_RESPONSE=$?
	
	check_platform_conditions "pre";
	
	if [ $IS_AGENT_INSTALLED = 0 ] ; then
		if [ $INST_FROM_RESPONSE = 1 ] ; then
			trace "Installing with parameters' values provided in response file"
			load_response;
		else
			trace "Installing with default parameters' values"
		fi
	else
		if [ $INST_FROM_RESPONSE = 1 ] ; then
			trace "Skipping response file in refresh or upgrade scenario"
			msg "nativeAgt.skipResponseFile"
			trace "Checking disableRollBack parameter"
			load_response "$DIS_ROLL_BACK_NAME";
		fi
	fi
	prepare_install_agent;
	resolve_conf;
	check_platform_conditions "post";
	install_agent;
	is_agent_installed_cleanup;
	trace "Exiting main()"
	trace "Installation successful"
	msg "nativeAgt.successfullyInstalled"
}

# Include logger script
. "${PARAMS[AGT_PKG_DIR]}/inst_agt_logger.sh"
# Initialize TCD logger
init_logging "${PARAMS[AGT_TEMP_DIR]}"

main;
